package com.sharry.lib.album;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.List;

/**
 * 图片文件夹的 Adapter
 *
 * @author Sharry <a href="SharryChooCHN@Gmail.com">Contact me.</a>
 * @version 1.0
 * @since 2018/9/22 15:02
 */
class FolderAdapter extends RecyclerView.Adapter<FolderAdapter.ViewHolder> {

    final Context context;
    final List<FolderModel> data;
    final AdapterInteraction callback;

    FolderAdapter(Context context, List<FolderModel> data) {
        if (context instanceof AdapterInteraction) {
            this.callback = (AdapterInteraction) context;
        } else {
            throw new IllegalArgumentException(context + "must implements " +
                    FolderAdapter.class.getSimpleName() + ".Interaction");
        }
        this.context = context;
        this.data = data;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(
                R.layout.lib_album_recycle_item_folder, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull final ViewHolder holder, int position) {
        FolderModel folder = data.get(holder.getAdapterPosition());
        if (folder == null || folder.getMetas() == null || folder.getMetas().isEmpty()) {
            return;
        }
        MediaMeta firstMeta = folder.getMetas().get(0);
        if (firstMeta.isPicture) {
            Loader.loadPicture(context, firstMeta, holder.ivPreview);
        } else {
            Loader.loadVideo(context, firstMeta, holder.ivPreview);
        }
        holder.tvFolderName.setText(folder.getName());
    }

    @Override
    public int getItemCount() {
        return data.size();
    }

    /**
     * Communicate with Activity.
     */
    public interface AdapterInteraction {

        void onFolderChecked(int position);

    }

    class ViewHolder extends RecyclerView.ViewHolder {
        private ImageView ivPreview;
        private TextView tvFolderName;

        private ViewHolder(View itemView) {
            super(itemView);
            tvFolderName = itemView.findViewById(R.id.tv_folder_name);
            ivPreview = itemView.findViewById(R.id.iv_preview);
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    callback.onFolderChecked(getAdapterPosition());
                }
            });
        }
    }

}
